==> linuxconf-tools-1.1.src.tar.gz: linuxconf-tools-1.1/projetx/livre.cc <==
/*
	Opration de livraison d'une srie de projet.
	La livraison, c'est l'inscription des modifications  un projet
	dans son archive.

	La livraison se fait en deux tapes:

	1- On doit documenter les changements.

		Un fichier change.log contient les changements pour tous les
		fichiers d'un projet.

	2- On archive.

		Cette opration est batch. On minimise le temps d'accs  l'archive
		pour limiter les chances qu'un crash viennent foutre le bordel.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"

/*
	Livraison d'un projet et optionnellement ses sous-rpertoires.
	Le projet "" est la racine de tous les projets. Il est livrable
	sauf que les sous-projets ne sont pas livrs rcursivement.

	Retourne le nombre de fichier livrs. -1 si erreur.
	Si retourne 0, cela signifie qu'il n'y avait rien  livr, donc la
	livraison du makefile.dat devient optionnelle. Ca vite l'archivage
	inutile du makefile.dat.
*/
int livre_revision (
	USERINFO *user,
	PRJCTRL *ctrl,
	PROJET_LOG *log,		// Affiche le traitement  l'cran
	const char *projet,
	const char *defcom,		// Commentaire par dfaut, si absent de
							// change.dat
	REVISION_TYPE revtype)
{
	int ret = 0;
	char path[MAXSIZ_PATH];
	user->makusrpath(projet,path);
	char *tb[400];
	int nb = ctrl->dirsource (path,WILD_ALLFILE,tb,400);
	/* #Spcification: livraison / critres / livraison force
		La prsence d'un fichier makefile.old indique que le
		makefile.dat a t rvis soit manuellement, soit via
		le mcanisme de fusion-dition.
		Cela implique qu'il faut le livrer mme si aucune autre
		transaction n'a t faite dans le rpertoire (pas de
		fichier corrig, aucun ajout, etc ...
	*/
	char nameold[MAXSIZ_PATH];
	path_make (path,"makefile.old",nameold);
	if (file_exist (nameold)) ret = 1;
	// Mme s'il n'y a aucun fichier  traiter, il y a peut
	// tre des fichier qui ont t effac.
	{
		int err = 0;
		if (projet[0] != '\0'){
			// Livraison des sous-rpertoires
			// On livre les sous-rpertoires avant pour mettre  jour
			// le fichier makefile.dat. La valeur de ret indiquera
			// combien on a livr de sous-rpertoires.
			for (int i=0; i<nb && err != -1; i++){
				char *fichier = tb[i];
				if (ctrl->isdir(fichier)){
					log->printf ("Rpertoire :%s:\n",fichier);
					char sous_projet[MAXSIZ_PATH];
					path_make (projet,fichier,sous_projet);
					path_stripsep (sous_projet,sous_projet);
					err = livre_revision (user,ctrl,log,sous_projet,defcom
						,revtype);
					if (err != -1) ret += err;
					/* #Specification: livraison / critres
						Un projet est livrable entre autre si un de
						ses sous-rpertoires a t livrs, puisque
						son makefile.dat a t chang.
					*/
				}
			}
		}
		if (err != -1){
			MAKEFILE mkf(NULL,1,projet,user);
			CHANGE_DAT change("change.dat");
			/* #Specification: livraison / critres
				Un projet est livrable s'il y a des fichiers qui ont
				changs, des fichiers qui se sont ajouts ou effacs et
				des sous-rpertoires qui ont t ajouts ou limins.
			*/
			{
				// Compte le nombre de fichiers effacs. Affiche un message
				// pour chaque fichier.
				mkf.setiter();
				MAKEFILE_FILE *file;
				while ((file=mkf.iterdel(WILD_ALLFILE))!=NULL){
					// Attention: Un fichier peut avoir t effac
					// et recr ensuite.
					// Il ne faut pas de message dans ce cas.
					if (!mkf.locate(file->getnom())){
						ret++;	// Note qu'il y a eu au moins un changement
						char fpath[MAXSIZ_PATH];
						path_make (path,file->getnom(),fpath);
						log->printf ("Elimine %s\n",fpath);
					}
				}
			}
			for (int i=0; i<nb && err != -1; i++){
				char *fichier = tb[i];
				if (!ctrl->isdir(fichier)){
					const char *comment = change.getdoc (fichier);
					if (comment == NULL) comment = defcom;
					err = mkf.livre (fichier,revtype,comment);
					if (err != -1) ret++;
				}
			}
			if (err != -1 && projet[0] != '\0'){
				/*
					Vrifie que tout les sous-rpertoires mentionns dans
					dans makefile.dat existe toujours.
					Ce traitement n'est pas fait dans la racine parce
					l'usager n'a pas ncessairement pris tous les projets.
				*/
				mkf.setiter();
				MAKEFILE_FILE *file;
				while ((file=mkf.iter("*/makefile.dat"))!=NULL){
					char dir[MAXSIZ_PATH];
					path_splitlex (file->getnom(),dir,NULL);
					path_make(path,dir,dir);
					/*
						Les sous-rpertoires h et doc sont limins
						de makefile.dat, mme s'il n'aurait jamais
						du tre prsent. Ca corrige un bug lors de
						la cration de l'archive.
					*/
					if (file_type (dir) != 1
						|| strcmp(dir,"h")==0
						|| strcmp(dir,"doc")==0){
						mkf.delfil(file->getnom());
						ret++;		// Note au moins un changement
									// dans makefile.dat
						log->printf ("Sous-Rpertoire %s n'existe plus\n",dir);
					}
				}
			}
			/* #Spcification: archivage / comportement en cas d'erreur.
				Si un problme survient lors de l'archivage d'un projet,
				le comportement est le suivant:

				Si au moins une composante du projet a t archive
				correctement, le projet sera archive comme si tout tait
				ok. livre_revision() retourne > 0.

				Si aucune composante n'a t archiv, livre_revision()
				retourne -1, indiquant un erreur.

				Ce comportement fait que mme un archivage partiel
				va laisser l'archive et le rpertoire usager dans un
				tat consistent.
			*/
			if (ret > 0 || projet[0] == '\0'){
				err = mkf.livretoi (revtype,1);
				if (err != -1){
					/* #Specification: livraison / succes

						Le fichier change.dat est renomm  change.old
						aprs une livraison ok. Ce fichier contient la
						documentation associe  chaque fichier livr.

						Le fichier makefile.old est effac. Ca vite
						que le makefile.dat soit archiv pour rien
						lors des prochaines livraisons.
					*/
					char change_dat[MAXSIZ_PATH];
					path_make (path,"change.dat",change_dat);
					char change_old[MAXSIZ_PATH];
					path_make (path,"change.old",change_old);
					file_unlink (change_old);
					file_rename (change_dat,change_old);
					file_unlink (nameold);
				}else{
					/*
						Impossible d'archiver le makefile.dat. L'archive
						est maintenant dans un tat inconsistente.
						On retourne -1 pour limiter les dgats.
					*/
					ret = -1;
					/* #Spcification: archivage / RCS / rparation manuelle
						Lorsqu'un archivage a chou compltement dans
						un rpertoire, il faut limins les rvisions
						de l'archive. Pour une rvision X.Y  liminer,
						les oprations suivantes:

						rcmd rcs -oX.Y %s
						edition de version.dat pour eliminer l'information
						associ  la rvision X.Y.
					*/
				}
			}
		}
		if (err == -1 && ret == 0) ret = -1;
	}
	tbstr_free (tb,nb);
	return ret;
}

/*
	Test de livraison d'un projet et optionnellement ses sous-rpertoires.
	Le projet "" est la racine de tous les projets. Il est livrable
	sauf que les sous-projets ne sont pas livrs rcursivement.

	Retourne le nombre d'erreur collecte.

	Test que tous les fichiers  archiver ont de la documentation.
*/
/* #Spcification: archivage / fichier binaire
	L'archivage de fichier binaire est pleinement support. Les commandes
	de diff (intgration) devront tre "protg". Le module ascii.c permet
	de reconnaitre de faon heuristique les fichiers binaires.
*/
int livre_test (
	USERINFO *user,
	PRJCTRL *ctrl,
	const char *projet,
	int test_comment,		// Vrifie s'il y a de la doc
	char *tberr[])			// Contiendra la liste des fichiers
							// pas accept.
{
	int nberr = 0;
	char path[MAXSIZ_PATH];
	user->makusrpath(projet,path);
	char *tb[400];
	int nb = ctrl->dirsource (path,WILD_ALLFILE,tb,400);
	if (nb > 0){
		if (projet[0] != '\0'){
			// Test de livraison des sous-rpertoires
			for (int i=0; i<nb; i++){
				char *fichier = tb[i];
				if (ctrl->isdir(fichier)){
					char sous_projet[MAXSIZ_PATH];
					path_make (projet,fichier,sous_projet);
					path_stripsep (sous_projet,sous_projet);
					nberr += livre_test (user,ctrl,sous_projet,test_comment
						,tberr+nberr);
				}
			}
		}
		//MAKEFILE mkf(NULL,1,projet,user);
		CHANGE_DAT change("change.dat");
		{
			for (int i=0; i<nb; i++){
				char *fichier = tb[i];
				char fpath[MAXSIZ_PATH];
				path_make (path,fichier,fpath);
				if (!ctrl->isdir(fichier)){
					const char *comment = change.getdoc (fichier);
					if (test_comment && comment == NULL){
						// Pas de documentation
						tberr[nberr++] = strdup_err (fpath);
					}
#if 0
					else if (ascii_test(fpath)==-1){
						// Pas un fichier ascii
						tberr[nberr++] = strdup_err (fpath);
					}
#endif
				}
			}
		}
	}
	tbstr_free (tb,nb);
	return nberr;
}

==> linuxconf-tools-1.7.src.tar.gz: linuxconf-tools-1.7/projetx/livre.cc <==
/*
	Opration de livraison d'une srie de projet.
	La livraison, c'est l'inscription des modifications  un projet
	dans son archive.

	La livraison se fait en deux tapes:

	1- On doit documenter les changements.

		Un fichier change.log contient les changements pour tous les
		fichiers d'un projet.

	2- On archive.

		Cette opration est batch. On minimise le temps d'accs  l'archive
		pour limiter les chances qu'un crash viennent foutre le bordel.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"

/*
	Livraison d'un projet et optionnellement ses sous-rpertoires.
	Le projet "" est la racine de tous les projets. Il est livrable
	sauf que les sous-projets ne sont pas livrs rcursivement.

	Retourne le nombre de fichier livrs. -1 si erreur.
	Si retourne 0, cela signifie qu'il n'y avait rien  livr, donc la
	livraison du makefile.dat devient optionnelle. Ca vite l'archivage
	inutile du makefile.dat.
*/
int livre_revision (
	USERINFO *user,
	PRJCTRL *ctrl,
	PROJET_LOG *log,		// Affiche le traitement  l'cran
	const char *projet,
	const char *comment,	// Commentaire  placer dans le version.dat
	REVISION_TYPE revtype,
	const char *version,
	const SSTRINGS &tbnew)	// Contiendra la liste des nouveaux fichiers
							// (jamais archiv)
{
	int ret = 0;
	char path[MAXSIZ_PATH];
	user->makusrpath(projet,path);
	SSTRINGS tb;
	int nb = ctrl->dirsource (path,WILD_ALLFILE,tb);
	/* #Spcification: livraison / critres / livraison force
		La prsence d'un fichier makefile.old indique que le
		makefile.dat a t rvis soit manuellement, soit via
		le mcanisme de fusion-dition.
		Cela implique qu'il faut le livrer mme si aucune autre
		transaction n'a t faite dans le rpertoire (pas de
		fichier corrig, aucun ajout, etc ...
	*/
	char nameold[MAXSIZ_PATH];
	path_make (path,"makefile.old",nameold);
	if (file_exist (nameold)) ret = 1;
	// Mme s'il n'y a aucun fichier  traiter, il y a peut
	// tre des fichier qui ont t effac.
	{
		int err = 0;
		if (projet[0] != '\0'){
			// Livraison des sous-rpertoires
			// On livre les sous-rpertoires avant pour mettre  jour
			// le fichier makefile.dat. La valeur de ret indiquera
			// combien on a livr de sous-rpertoires.
			for (int i=0; i<nb && err != -1; i++){
				const char *fichier = tb.getitem(i)->get();
				if (ctrl->isdir(fichier)){
					log->printf ("Rpertoire :%s:\n",fichier);
					char sous_projet[MAXSIZ_PATH];
					path_make (projet,fichier,sous_projet);
					path_stripsep (sous_projet,sous_projet);
					err = livre_revision (user,ctrl,log,sous_projet,comment
						,revtype,version,tbnew);
					if (err != -1) ret += err;
					/* #Specification: livraison / critres
						Un projet est livrable entre autre si un de
						ses sous-rpertoires a t livrs, puisque
						son makefile.dat a t chang.
					*/
				}
			}
		}
		if (err != -1){
			MAKEFILE mkf(NULL,1,projet,user);
			CHANGE_DAT change("change.dat");
			/* #Specification: livraison / critres
				Un projet est livrable s'il y a des fichiers qui ont
				changs, des fichiers qui se sont ajouts ou effacs et
				des sous-rpertoires qui ont t ajouts ou limins.
			*/
			{
				// Compte le nombre de fichiers effacs. Affiche un message
				// pour chaque fichier.
				mkf.setiter();
				MAKEFILE_FILE *file;
				while ((file=mkf.iterdel(WILD_ALLFILE))!=NULL){
					// Attention: Un fichier peut avoir t effac
					// et recr ensuite.
					// Il ne faut pas de message dans ce cas.
					if (!mkf.locate(file->getnom())){
						ret++;	// Note qu'il y a eu au moins un changement
						char fpath[MAXSIZ_PATH];
						path_make (path,file->getnom(),fpath);
						log->printf ("Elimine %s\n",fpath);
					}
				}
			}
			for (int i=0; i<nb && err != -1; i++){
				const char *fichier = tb.getitem(i)->get();
				if (!ctrl->isdir(fichier)){
					const char *doc = change.getdoc (fichier);
					bool doarch = false;
					err = mkf.livre (fichier,revtype,doc,tbnew,doarch);
					if (doarch) ret++;
				}
			}
			if (err != -1 && projet[0] != '\0'){
				/*
					Vrifie que tout les sous-rpertoires mentionns dans
					dans makefile.dat existe toujours.
					Ce traitement n'est pas fait dans la racine parce
					l'usager n'a pas ncessairement pris tous les projets.
				*/
				mkf.setiter();
				MAKEFILE_FILE *file;
				while ((file=mkf.iter("*/makefile.dat"))!=NULL){
					char dir[MAXSIZ_PATH];
					path_splitlex (file->getnom(),dir,NULL);
					path_make(path,dir,dir);
					/*
						Dsuet

						Les sous-rpertoires h et doc sont limins
						de makefile.dat, mme s'il n'aurait jamais
						du tre prsent. Ca corrige un bug lors de
						la cration de l'archive.
						|| strcmp(dir,"h")==0
						|| strcmp(dir,"doc")==0
					*/
					if (file_type (dir) != 1){
						mkf.delfil(file->getnom());
						ret++;		// Note au moins un changement
									// dans makefile.dat
						log->printf ("Sous-Rpertoire %s n'existe plus\n",dir);
					}
				}
			}
			/* #Spcification: archivage / comportement en cas d'erreur.
				Si un problme survient lors de l'archivage d'un projet,
				le comportement est le suivant:

				Si au moins une composante du projet a t archive
				correctement, le projet sera archive comme si tout tait
				ok. livre_revision() retourne > 0.

				Si aucune composante n'a t archiv, livre_revision()
				retourne -1, indiquant un erreur.

				Ce comportement fait que mme un archivage partiel
				va laisser l'archive et le rpertoire usager dans un
				tat consistent.
			*/
			if (ret > 0 || projet[0] == '\0'){
				err = mkf.livretoi (revtype,true,version,comment);
				if (err != -1){
					/* #Specification: livraison / succes

						Le fichier change.dat est renomm  change.old
						aprs une livraison ok. Ce fichier contient la
						documentation associe  chaque fichier livr.

						Le fichier makefile.old est effac. Ca vite
						que le makefile.dat soit archiv pour rien
						lors des prochaines livraisons.
					*/
					char change_dat[MAXSIZ_PATH];
					path_make (path,"change.dat",change_dat);
					char change_old[MAXSIZ_PATH];
					path_make (path,"change.old",change_old);
					file_unlink (change_old);
					file_rename (change_dat,change_old);
					file_unlink (nameold);
				}else{
					/*
						Impossible d'archiver le makefile.dat. L'archive
						est maintenant dans un tat inconsistente.
						On retourne -1 pour limiter les dgats.
					*/
					ret = -1;
					/* #Spcification: archivage / RCS / rparation manuelle
						Lorsqu'un archivage a chou compltement dans
						un rpertoire, il faut limins les rvisions
						de l'archive. Pour une rvision X.Y  liminer,
						les oprations suivantes:

						rcmd rcs -oX.Y %s
						edition de version.dat pour eliminer l'information
						associ  la rvision X.Y.
					*/
				}
			}else if (ret == 0){
				/* #Specification: archivage / fichier non modifi
					Mme si aucun fichier n'est archiv dans un rpertoire
					il y a quand mme des fichiers qui seront effac parce
					que la version dans l'archive est identique. Donc
					mme une livraison qui ne livre rien, fait du mnage
				*/
				mkf.cleanup();
			}
		}
		if (err == -1 && ret == 0) ret = -1;
	}
	return ret;
}
/* #Spcification: archivage / fichier binaire
	L'archivage de fichier binaire est pleinement support. Les commandes
	de diff (intgration) devront tre "protg". Le module ascii.c permet
	de reconnaitre de faon heuristique les fichiers binaires.
*/
#
/*
	Test de livraison d'un projet et optionnellement ses sous-rpertoires.
	Le projet "" est la racine de tous les projets. Il est livrable
	sauf que les sous-projets ne sont pas livrs rcursivement.

	Retourne le nombre d'erreur collecte.

	Test que tous les fichiers  archiver ont de la documentation.
*/
int livre_test (
	USERINFO *user,
	PRJCTRL *ctrl,
	const char *projet,
	bool test_comment,		// Vrifie s'il y a de la doc
	SSTRINGS &tberr,		// Contiendra la liste des fichiers
							// pas accept.
	SSTRINGS &tbnew)		// Contiendra la liste des nouveaux fichiers
							// (jamais archiv)
{
	int nberr = 0;
	char path[MAXSIZ_PATH];
	user->makusrpath(projet,path);
	SSTRINGS tb;
	int nb = ctrl->dirsource (path,WILD_ALLFILE,tb);
	if (nb > 0){
		if (projet[0] != '\0'){
			// Test de livraison des sous-rpertoires
			for (int i=0; i<nb; i++){
				const char *fichier = tb.getitem(i)->get();
				if (ctrl->isdir(fichier)){
					char sous_projet[MAXSIZ_PATH];
					path_make (projet,fichier,sous_projet);
					path_stripsep (sous_projet,sous_projet);
					nberr += livre_test (user,ctrl,sous_projet,test_comment
						,tberr,tbnew);
				}
			}
		}
		MAKEFILE mkf(NULL,true,projet,user);
		for (int i=0; i<nb; i++){
			const char *fichier = tb.getitem(i)->get();
			if (!ctrl->isdir(fichier)){
				MAKEFILE_FILE *file = mkf.locate (fichier,true);
				if (file == NULL){
					// Pas dans l'archive
					char fpath[MAXSIZ_PATH];
					path_make (projet,fichier,fpath);
					tbnew.add (new SSTRING(fpath));
				}
			}
		}
		if (test_comment){
			CHANGE_DAT change("change.dat");
			for (int i=0; i<nb; i++){
				const char *fichier = tb.getitem(i)->get();
				if (!ctrl->isdir(fichier)){
					const char *comment = change.getdoc (fichier);
					if (comment == NULL){
						// Pas de documentation
						char fpath[MAXSIZ_PATH];
						path_make (path,fichier,fpath);
						tberr.add (new SSTRING(fpath));
					}
				}
			}
		}
	}
	return nberr;
}
